/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "dac2.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define DAC2_DUMMY 0

#define DAC2_PERCENTAGE_VALUE            100
#define DAC2_16_BIT_DATA_VALUE           65535
#define DAC2_DIV_FACTOR                  100
#define DAC2_DATA_MASK                   0x00FF

// -------------------------------------------------------------- PRIVATE TYPES


// ------------------------------------------------------------------ CONSTANTS


// ------------------------------------------------------------------ VARIABLES


// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void dev_comm_delay ( void );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void dac2_cfg_setup ( dac2_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck  = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs   = HAL_PIN_NC;

    // Additional gpio pins

    cfg->clr = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode  = SPI_MODE_0;
}

DAC2_RETVAL dac2_init ( dac2_t *ctx, dac2_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return DAC2_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, DAC2_DUMMY );

    // Output pins 
    
    digital_out_init( &ctx->clr, cfg->clr );

    return DAC2_OK;

}

void dac2_generic_transfer ( dac2_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi );
    dev_comm_delay( );
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );    
}

void dac2_default_cfg ( dac2_t *ctx )
{
    // Reset LTC2601
    
    dac2_clr_dac_2_reg( ctx );
    dev_comm_delay( );
}

void dac2_clr_dac_2_reg ( dac2_t *ctx )
{
    digital_out_low( &ctx->clr );
    dev_comm_delay( );
    digital_out_high( &ctx->clr );
}

void dac2_write_output_voltage_procentage ( dac2_t *ctx, uint8_t value_pct )
{
    spi_master_transfer_data_t block;
    
    uint16_t value_output;
    uint8_t tx_buf[ 2 ];
    uint8_t rx_buf;
    
    dac2_clr_dac_2_reg( ctx );

    if ( value_pct > 100 )
    {
        value_pct %= DAC2_PERCENTAGE_VALUE;
    }

    value_output   = ( DAC2_16_BIT_DATA_VALUE / DAC2_DIV_FACTOR );
    value_output  *= value_pct;
    
    tx_buf[ 0 ]    = value_output >> 8;
    tx_buf[ 1 ]    = value_output & DAC2_DATA_MASK;
    
    block.tx_buffer = tx_buf;
    block.tx_length = 2;
    block.rx_buffer = &rx_buf;
    block.rx_length = 0;
    
    dac2_generic_transfer( ctx, &block );
}

void dac2_write_output_voltage_data ( dac2_t *ctx, uint16_t value_data )
{
    spi_master_transfer_data_t block;
    
    uint8_t tx_buf[ 2 ];
    uint8_t rx_buf;
    
    dac2_clr_dac_2_reg( ctx );

    tx_buf[ 0 ] = value_data >> 8;
    tx_buf[ 1 ] = value_data  & DAC2_DATA_MASK;
    
    block.tx_buffer = tx_buf;
    block.tx_length = 2;
    block.rx_buffer = &rx_buf;
    block.rx_length = 0;
    
    dac2_generic_transfer( ctx, &block );
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void dev_comm_delay ( void )
{
    Delay_10us();
}

// ------------------------------------------------------------------------- END

